##
## jabberif base classes
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

class handler:
	"""handler class - for future use"""
	## currently there is only one handler. the plugin handler.
	pass

class plugin:
	## required attributes
	_name = "unknown" ## exact plugin class name
	_type = "unknown" ## type of plugin; one of 'core', 'input', 'output'
	_desc = "unknown" ## short description of the plugin
	# ph ## the plugin handler - must be set in __init__() 
	_exit = False ##

	def _on_load(self):
		"""init function - called directly after __init__()"""
		pass
	def _on_unload(self):
		"""destructor?!? - called before progam exit and on module unload"""
		pass

	## return required attributes
	def _get_name(self):
		return self._name
	def _get_type(self):
		return self._type
	def _get_desc(self):
		return self._desc

	## reload configuration after configuration file has been reparsed
	def _configure(self):
		pass


class input_plugin(plugin):
	"""input plugin
		after program startup process() is called indefinitely until program exit"""
		
	_type = 'input'
	_inputtype = 'unknown'

	def process(self):
		"""gather input and redirect it to the command processor"""
		pass
		
	def _send_back(self, msg, message):
		"""send message back to where it came from
			params:
				msg: helpers.message object
				message: message string"""
		pass
	
	def _send(self, id, message):
		"""send message to id"""
		return "error: cannot send messages with this handler"
	
	def _is_available(self, id):
		"""check a users online status"""
		return False
	

class core_plugin(plugin):
	"""core plugin - to be used only by plugins which are interferring directly into
		the pluginhandler internals"""
	_type = 'core'

class output_plugin(plugin):
	"""output plugin - use this type for all other plugins not mentioned before"""
	_type = 'output'

