##
## jabberif amazon plugin
##	keyword search in amazon.com
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import output_plugin
from string import join
import SOAPpy

class amazon_plugin(output_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'amazon_plugin'
		self._desc = 'keyword search in amazon.com'

		self.__default_config = { 'amazon_developers_token':'',
					'amazon_associates_tag':'webservices-20',
					'http_proxy': None }
		self._configure()
		
		## init amazon soap proxy
		url = 'http://soap.amazon.com/schemas3/AmazonWebServices.wsdl'
		self.server = SOAPpy.WSDL.Proxy(url, http_proxy=self.__config['http_proxy'])
		

	
	def _configure(self):
		self.__config = self.ph.get_plugin_config(self, self.__default_config)
		self.__developers_token = self.__config['amazon_developers_token']
		self.__associates_tag = self.__config['amazon_associates_tag']
	
	def _on_load(self):
		self.ph._register_command(self, 'amazon', self.keywordsearch, '[keywords] search in amazon')
	
	
	####################################################################################################
	## /amazon command
	
	def keywordsearch(self, args):
		##KeywordSearchRequest
		##
		##      keyword  string
		##      page  string
		##      mode  string
		##      tag  string
		##      type  string
		##      devtag  string
		##      sort  string  (optional)
		##      locale  string  (optional, defaults to US)
		##      price  string  (optional)

		try:
			data = self.server.KeywordSearchRequest({'keyword': join(args['cmd'][1:], ' '), 'page':'1', 'mode':'books',
				'tag':self.__associates_tag, 'type':'lite', 'devtag':self.__developers_token, 'sort': '+daterank'})
		except SOAPpy.Types.faultType, t:
			args['msg'].send_back(t.detail)
			return

		outs = []
		outs.append("results: "+ data.TotalResults)

		if data.TotalResults > 0:
			for book in [a._asdict() for a in data.Details][:5]:
				for item in ['Authors', 'ProductName', 'ReleaseDate', 'Asin', 'ListPrice']:
					if not book.has_key(item):
						book[item] = 'unknown'
				outs.append("\n%s: %s\n%s: %s\n%s: %s\n%s: %s\n%s: %s" %
					('title', book['ProductName'],
					'author[s]', join(book['Authors'], ', '),
					'date', book['ReleaseDate'],
					'asin', book['Asin'],
					'list price', book['ListPrice']))
		
		## send output back to sender
		output = join(outs, '\n')
		args['msg'].send_back('\n'+output)

