##
## jabberif base plugin
##	basic functionalities and plugins are handled here.
##	this is a core plugin and therefore should not be removed during runtime.
##
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import core_plugin
from string import join
import threading

class base_plugin(core_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'base_plugin'
		self._desc = 'core functions'

	def _on_load(self):
		self.ph._register_command(self, 'quit', self.quit, 'quit jabberif', { 'level': 0 })
		self.ph._register_command(self, 'tlist', self.threadlist, 'list threads', { 'level': 10 })
		self.ph._register_command(self, 'pload', self.load_plugins, '[pluginlist] load plugins', { 'level': 5 })
		self.ph._register_command(self, 'punload', self.unload_plugins, '[pluginlist] unload plugins', { 'level': 5 })
		self.ph._register_command(self, 'preload', self.reload_plugins, '[pluginlist] reload plugins', { 'level': 5 })
		self.ph._register_command(self, 'plist', self.list_plugins, 'list plugins', { 'level': 100 })
		self.ph._register_command(self, 'send', self.send_message, '[id, msg] send message', { 'level': 20 })
	
	####################################################################################################
	
	def quit(self, args):
		self.ph._quit()
	
	def threadlist(self, args):
		threadlist = [t.getName() for t in threading.enumerate()]
		args['msg'].send_back(join(threadlist, '\n'))
	
	####################################################################################################
	## plugin commands

	def load_plugins(self, args):
		send_back = args['msg'].send_back
		
		for plugin in args['cmd'][1:]:
			error = self.ph._load(plugin)
			if error:
				send_back(error)
			else:
				send_back('successfully loaded ' + plugin)
	
	def unload_plugins(self, args):
		send_back = args['msg'].send_back
		
		for plugin in args['cmd'][1:]:
			error = self.ph._unload(plugin)
			if error:
				send_back(error)
			else:
				send_back('successfully unloaded ' + plugin)
	
	def reload_plugins(self, args):
		send_back = args['msg'].send_back

		for plugin in args['cmd'][1:]:
			error = self.ph._unload(plugin)
			if not error:
				error = self.ph._load(plugin)
				
			if error:
				send_back(error)
			else:
				send_back('successfully reloaded ' + plugin)
	
	def list_plugins(self, args):
		names = self.ph._get_plugin_info()
		allnames = names.keys()
		allnames.sort()
		out = ["%s [%s] %s" % (a, names[a]['type'], names[a]['desc']) for a in allnames]
		args['msg'].send_back('\n'+join(out, '\n'))
		
		
	####################################################################################################
	## send message

	def send_message(self, args):
		params = args['cmd'][1:]
		if len(params) < 2:
			args['msg'].send_back('syntax error. see /help '+args['cmd'][0])
			return
		error = self.ph.send_message(params[0], join(params[1:], ' '))
		if error:
			args['msg'].send_back(error)
		else:
			args['msg'].send_back('message sent.')


