##
## jabberif help plugin
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import output_plugin
import re
from string import join

class help_plugin(output_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'help_plugin'
		self._desc = 'useful help'

		self.__default_config = { 'about_text':'jabberif - http://fuhrmannek.de' }
		self._configure()

	def _configure(self):
		self.__config = self.ph.get_plugin_config(self, self.__default_config)

	def _on_load(self):
		self.ph._register_command(self, 'help', self.help, '[cmd] show help', { 'longhelp':'n/a'})
		self.ph._register_command(self, 'about', self.about, 'about jabberif')
	

	####################################################################################################
	## /help command
	
	def help(self, args):
		"""show a list of available commands"""
		params = args['cmd'][1:]
		user_level = self.ph.get_userlevel(args['msg'].get_uid())
		
		out = []
		data = self.ph._get_command_info(params)
		if params: ## a command list has been specified as parameter
			params.sort()
			for param in params:
				if not data.has_key(param):
					out.append("command '%s' not found" % param)
				elif data[param]['level'] < user_level:
					out.append("command '%s' is restrictedby level %d" % (param, data[param]['level']))
				else:
					out.append(self.__longhelp(param, data[param]))
		else: ## list all available commands
			params = data.keys()
			params.sort()
			for param in params:
				if data[param]['level'] >= user_level:
					out.append(self.__shorthelp(param, data[param]))
			
		args['msg'].send_back('\n'+join(out, '\n'))
	
	def __shorthelp(self, name, data):
		"""format help data as short as possible"""
		return "[%3d] /%s %s" % (data['level'], name, data['help'])

	def __longhelp(self, name, data):
		"""format help data in a verbose and detailed way"""
		allhelp = re.split('[\]\[]', data['help'])
		if len(allhelp) < 2:
			params = 'none'
			helptext = data['help']
		else:
			sep = data['help'].find(']')+1
			params = data['help'][:sep]
			helptext = data['help'][sep:]

		description = data['longhelp']
		if not description:
			description = 'n/a'

		return "command: /%s\nlevel: %3d\nmodule: %s\nparams: %s\nhelp: %s\ndescription: %s\n" % (name, data['level'], data['module'], params, helptext, description)


	####################################################################################################
	## /about command
	
	def about(self, args):
		"""show about message"""
		args['msg'].send_back('\n'+self.__config['about_text'])

