##
## jabberif xmpp jabber input plugin
##	this plugin provides jabber input support.
##	it is a pelacement for the jabber_input_plugin
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import input_plugin
from helpers import message
from string import split,strip,join
import sys,os
import re
import socket


import xmpp


class xmpp_input_plugin(input_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'xmpp_input_plugin'
		self._desc = 'jabber access'
		self._inputtype = 'jabber'

		self.__default_config = {	'server':'jabber.org',
						'ssl':'on',
						'port':'5223',
						'username':'',
						'password':'',
						'resource':'jabberif',
						'priority':'3' }
		self._configure()

	def _configure(self):
		self.__config = self.ph.get_plugin_config(self, self.__default_config)

	def _on_load(self):
		self.connect()
		self.ph._register_command(self, 'julist', self.show_users, 'list jabber users', {'level':10})
		self.ph._register_command(self, 'jureload', self.reload_users, 'reload roaster', {'level':10})
		self.ph._register_command(self, 'jsendpr', self.send_presence, '[jid, type] send presence', {'level':10})
		self.ph._register_command(self, 'jsubscribe', self.send_subscribe, '[jid] send subscribe', {'level':10})

	def _on_unload(self):
		self.disconnect()
		
	####################################################################################################
	## callback functions
	
	def messageCB(self, con, msg):
		"""Called when a message is recieved"""
		if msg.getBody():
			msgbody = strip(msg.getBody())
			if len(msgbody) > 0:
				msg = message(self._inputtype, self, msg.getFrom().getStripped(), msgbody, args={'msg':msg})
				self.ph._process_request(msg=msg, threaded=True, dos_detection=True)

	def presenceCB(self, con, prs):
		"""Called when a presence is recieved"""
		who = prs.getFrom()
		type = prs.getType()
		if type == None: type = 'available'

		elif type == 'unsubscribe':
			self.__cl.Roster.Unauthorize(who)
			self.__cl.Roster.Unsubscribe(who)

		elif type == 'subscribe':
			self.__cl.Roster.Subscribe(who)
		

	def disconnectedCB(self, con):
		lh.log("Ouch, network error", 'error')
		

	####################################################################################################
	## connection
	
	def connect(self):
		connection = 1

		cl = xmpp.Client(self.__config['server'], debug=[])
		self.__cl = cl

		if not cl.connect(server=(self.__config['server'], int(self.__config['port']))):
			error = self.ph.log("Couldn't connect: %s" % e, 'error')
			return error

		self.ph.log("Connected",'connection')
		

		## handler registration
		cl.RegisterHandler('message', self.messageCB)
		cl.RegisterHandler('presence', self.presenceCB)
		cl.RegisterHandler('disconnect', self.disconnectedCB)


		## login
		self.ph.log("Attempting to log in...", 'connection')

		#print "login with %s,%s,%s" % (self.__config['username'], self.__config['password'], self.__config['resource'])
		if cl.auth(self.__config['username'], self.__config['password'], self.__config['resource']):
			self.ph.log("Logged in as %s to server %s" % ( self.__config['username'], self.__config['server']), 'connection')
		else:
			return self.ph.log("authentication failed", 'error')

		## roster/presence init
		self.ph.log("Requesting Roster Info" , 'connection')
		cl.sendInitPresence()
		cl.send(xmpp.protocol.Presence(priority=self.__config['priority']))

		self.JID = self.__config['username'] + '@' + self.__config['server'] + '/' + self.__config['resource']
		return None

	def disconnect(self):
		if hasattr(self, '__cl'):
			self.__cl.disconnect()


	####################################################################################################
	## input plugin functions

	def process(self):
		if not self.__cl.isConnected(): self.__cl.reconnectAndReauth()
		self.__cl.Process(1)
			
	
	def _send_back(self, msg, message):
		self._send(msg.get_from(), message)


	def _send(self, name, message):
		msg = xmpp.protocol.Message(to=name, typ='chat', body=message)
		try:
			self.__cl.send(msg)
		except:
			return "error sending jabber message"
		return None
	
	def _is_available(self, id):
		users = self.__cl.Roster._data
		if not id in users:
			return False
		if not users[id].has_key('resources'):
			return False
		for cl in users[id]['resources']:
			if users[id]['resources'][cl].has_key('show') and users[id]['resources'][cl]['show'] == 'online':
				return True
		## to be tested...
		return False
		
	####################################################################################################
	## command functions
	
	def show_users(self, args):
		users = self.__cl.Roster._data
		
		out = []
		for user in users.keys():
			out.append("%s:" % (user))
			for key in users[user].keys():
				out.append("\t%s: %s" % (key, users[user][key]))

		args['msg'].send_back('\n' + join(out, '\n'))

	def reload_users(self, args):
		self.__cl.sendInitPresence()
		args['msg'].send_back('roaster reloaded')

	def __send_presence(self, id, type):
		self.__cl.send(xmpp.protocol.Presence(to=id, typ=type))
		
	def send_presence(self, args):
		data = args['cmd'][1:]
		if len(data) != 2:
			args['msg'].send_back('syntax error. see /help')
			return
		id, type = data
		self.__send_presence(id, type)
		args['msg'].send_back("sent %s to %s" % (type, id))
	
	def send_subscribe(self, args):
		data = args['cmd'][1:]
		if len(data) != 1:
			args['msg'].send_back('syntax error. see /help')
			return
		id = data[0]
		self.__send_presence(id, 'subscribe')
		self.__send_presence(id, 'subscribed')
		args['msg'].send_back("subscribed %s" % id)
	
